------------------------------------------------------------------------
-- Script:			import-profile-data.sql
-- Author:			Davide Mauri (SolidQ)
-- Version:         16
-- Credits:			-
-- Copyright:		Attribution-NonCommercial-ShareAlike 2.5
-- Target Version:	SQL Server 2005 or above
-- Tab/indent size:	4
-- Notes:			-
------------------------------------------------------------------------
set nocount on
set quoted_identifier on
go

print 'Import Profile Data Batch v. 16'
go

:setvar required_db_version 15

IF (LEN('$(project_name)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "project_name" must be specified.',16,127) WITH NOWAIT
END
GO

IF (LEN('$(section_name)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "section_name" must be specified.',16,127) WITH NOWAIT
END
GO

IF (LEN('$(dtloggedexec_db_name)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "dtloggedexec_db_name" must be specified.',16,127) WITH NOWAIT
END
GO

IF (LEN('$(dataflowprofile_path)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "dataflowprofile_path" must be specified.',16,127) WITH NOWAIT
END
GO

IF (LEN('$(dataflowprofile_file)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "dataflowprofile_file" must be specified.',16,127) WITH NOWAIT
END
GO

use [$(dtloggedexec_db_name)]
go

print 'Logs are bound to project "$(project_name)", section "$(section_name)"'
print 'Reading file "$(dataflowprofile_path)$(dataflowprofile_file)"'
print 'Using format file "$(dataflowprofile_path)formatheader.fmt"'
go

/*

	check for compatibility

*/
--Check that db has been updated to the correct schema version
declare @dbschemaversion int;
select top 1 @dbschemaversion = schema_version from dbo.[version] order by version_date desc;

if (@dbschemaversion < $(required_db_version))
begin
	raiserror ('DTLoggedExecDB must be updated to schema version $(required_db_version) or later.', 16, 127) with nowait
end
go

--Get File Type Version
declare @fileVersion as int;
declare @readedFileVersion as varchar(100);
with cte as
(
select 
	[field] = substring(Header, charindex('#', Header) + 1, charindex(': ', Header) - charindex('#', Header) - 1),
	[value] = substring(Header, charindex(': ', Header) + 2, 512)
from 
	openrowset
	(
		bulk '$(dataflowprofile_path)$(dataflowprofile_file)', 
		formatfile='$(dataflowprofile_path)formatheader.fmt', 
		firstrow = 0, lastrow = 1
	) t
)
select
	@readedFileVersion = [value]
from
	cte
where
	[field] = '__DataFileVersion'
;

begin try
	set @fileVersion = isnull(cast(@readedFileVersion as int), 1)
end try
begin catch
	set @fileVersion = 1
end catch

if (@fileVersion < 3) 
begin
	raiserror ('Loaded file must have a version 3 header at least to continue.', 16, 127) with nowait
end
go
	
	
/*

	Read Header Data

*/
print 'Loading header data...'
if (object_id('tempdb..#header') is not null) drop table #header;

with cte as
(
select 
	[field] = substring(Header, charindex('#', Header) + 1, charindex(': ', Header) - charindex('#', Header) - 1),
	[value] = substring(Header, charindex(': ', Header) + 2, 512)
from 
	openrowset
	(
		bulk '$(dataflowprofile_path)$(dataflowprofile_file)', 
		formatfile='$(dataflowprofile_path)formatheader.fmt', 
		firstrow = 2, lastrow = 9
	) t
)
select
	[package],
	[version],
	[package_guid] = cast(substring(PackageGUID, 2, 36) as uniqueidentifier),
	[version_guid] = cast(substring(VersionGUID, 2, 36) as uniqueidentifier),
	[execution_guid] = cast(substring(ExecutionGUID, 2, 36) as uniqueidentifier),
	[user_name] = UserName,
	[machine_name] = MachineName,
	[execution_datetime] = cast(ExecutionDateTime as datetime)
into
	#header
from
	cte
pivot
	(
		max(value) for [field] in (Package, Version, PackageGUID, VersionGUID, ExecutionGUID, UserName, MachineName, ExecutionDateTime)
	) p;



/*
	Save project data into dbo.projects table, or load project_id value
	if project already exists
*/
declare @project_id int;
set @project_id = (select project_id from [dbo].[projects] p where p.name = '$(project_name)')

if (@project_id is null) begin
	insert into dbo.[projects] ([name]) values ('$(project_name)');
	
	set @project_id = scope_identity()
end


/*
	Save section data into dbo.section, or load project_id value
	if project already exists
*/
declare @section_id int;
set @section_id = (select section_id from [dbo].[sections] s where s.name = '$(section_name)')

if (@section_id is null) begin
	insert into dbo.[sections] ([name]) values ('$(section_name)');
	
	set @section_id = scope_identity()
end


/*

	Save package data into dbo.packages table, or load package_id value 
	if package data already exists

*/
declare @package_id int;

set @package_id = (select package_id from [dbo].[packages] p inner join #header h on p.package_guid = h.package_guid and p.version_guid = h.version_guid)

if (@package_id is null) begin
	insert into dbo.[packages] 
	select top 1 [package], [version], [package_guid], [version_guid], @project_id, @section_id from #header;
	
	set @package_id = scope_identity()
end else begin
	update dbo.[packages] set project_id = @project_id, section_id = @section_id where package_id = @package_id
end

/*

	Save package execution data into dbo.package_executions

*/
declare @hasError int
set @hasError = 0
begin try
	begin tran

	declare @execution_id int;

	set @execution_id = (select package_execution_id from [dbo].[package_executions] p inner join #header h on p.execution_guid = h.execution_guid and p.package_id = @package_id)

	if (@execution_id is not null) begin
		if (exists(select top 1 * from [dbo].[dataflow_profiling] where package_execution_id = @execution_id)) begin
			drop table #header		
			raiserror ('DataFlow Profiling Data already loaded', 16, 127) with nowait
		end
	end

	if (@execution_id is null) begin
		insert into dbo.[package_executions] ([package_id], [user_name], [machine_name], [execution_datetime], [execution_guid]) 
		select top 1 @package_id, [user_name], [machine_name], [execution_datetime], [execution_guid] from [#header] as h

		set @execution_id = scope_identity()
	end


	/*

		Save package execution profiling datetime into dbo.dataflow_profiling table

	*/
	declare @profile_id int;

	begin try
		insert into dbo.[dataflow_profiling] ([package_execution_id], [execution_datetime]) 
		select top 1 @execution_id, [execution_datetime] from [#header] as h

		set @profile_id = scope_identity()
	end try
	begin catch
		if (@profile_id is null) begin
			drop table #header
			raiserror ('DataFlow Profiling data already loaded', 16, 127) with nowait
		end
	end catch


	/*

		Save package execution profiling data into dataflow_profiling_buffers_data table

	*/
	print 'Loading profiled data...'

	insert into
		[dbo].[dataflow_profiling_buffers_data]
	select 
		@profile_id,
		DataFlowId,
		DataFlowName,
		ExecutionNumber = cast(ExecutionNumber as int),
		SourceId,
		SourceName,
		ComponentId,
		ComponentName,
		DestinationId,
		DestinationName,	 
		[RowCount] = cast([RowCount] as bigint),
		[EventDateTime] = cast([EventDate] + ' ' + left([EventTime], 12) as datetime)
	from 
		openrowset
		(
			bulk '$(dataflowprofile_path)$(dataflowprofile_file)', 
			formatfile='$(dataflowprofile_path)formatfile.fmt', 
			firstrow = 2
		) t;
		
		
	/*

		Aggregate imported data over components, for faster queries

	*/

	print 'Aggregating (1 of 2)...'

	insert into 
		[dbo].[dataflow_profiling_components_data]	
	select
		[profile_id],
		[dataflow_id],
		[dataflow_name],
		[execution_count],
		[source_id],
		[source_name],
		[component_id],
		[component_name],
		[destination_id],
		[destination_name],
		[execution_start] = min([event_datetime]),
		[execution_end] = max([event_datetime]),
		[execution_elapsed_seconds] = datediff(ss, min([event_datetime]), max([event_datetime])),
		[processed_rows] = sum(row_count)
	from
		dbo.[dataflow_profiling_buffers_data] as dpd 
	where 
		[profile_id] = @profile_id
	group by
		[profile_id],
		[dataflow_id],
		[dataflow_name],
		[execution_count],
		[source_id],
		[source_name],
		[component_id],
		[component_name],
		[destination_id],
		[destination_name]	


	/*

		Aggregate imported data over dataflow, for faster queries

	*/

	print 'Aggregating (2 of 2)...'

	insert into 
		[dbo].[dataflow_profiling_dataflows_data]	
	select
		[profile_id],
		[dataflow_id],
		[dataflow_name],
		[execution_count],	
		[execution_start] = min([event_datetime]),
		[execution_end] = max([event_datetime]),
		[execution_elapsed_seconds] = datediff(ss, min([event_datetime]), max([event_datetime])),
		[processed_rows] = sum(row_count)
	from
		dbo.[dataflow_profiling_buffers_data] as dpd 
	where 
		[profile_id] = @profile_id
	group by
		[profile_id],
		[dataflow_id],
		[dataflow_name],
		[execution_count]	
		
		commit tran
end try
begin catch
	print 'ERROR FOUND: ' + error_message();
	if (@@trancount > 0) rollback tran;	
	set @hasError = 1
end catch	
	
/*

	Cleanup
	
*/

print 'Cleaning Up...'

drop table #header;

if (@hasError = 0) 
begin
	print 'Done'
end else begin
	raiserror ('Batch Executed With Errors', 16, 127)  with nowait
end