/*

Update script for DTLoggedExecDB

*/

:setvar required_db_version "18"
:setvar schema_version "19"
:setvar version_date "20110912"

SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, CONCAT_NULL_YIELDS_NULL, QUOTED_IDENTIFIER ON;
GO

SET NUMERIC_ROUNDABORT OFF;
GO

USE [master]
GO

:on error exit
GO

/*

	Check Parameters 

*/

IF (LEN('$(DatabaseName)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "DatabaseName" must be specified.',16,127) WITH NOWAIT
END
go

/*

	Check Database 

*/
IF (DB_ID(N'$(DatabaseName)') IS null) begin
	RAISERROR('Database [$(DatabaseName)] does not exists or is not accesible.',16,127) WITH NOWAIT
end
go

IF (DB_ID(N'$(DatabaseName)') IS NOT NULL
    AND DATABASEPROPERTYEX(N'$(DatabaseName)','Status') <> N'ONLINE')
BEGIN
    RAISERROR(N'The state of the target database, %s, is not set to ONLINE. To deploy to this database, its state must be set to ONLINE.', 16, 127,N'$(DatabaseName)') WITH NOWAIT
    RETURN
END

use [$(DatabaseName)]
go

/*

	Check for compatibility

*/
--Check that db has been updated to the correct schema version
declare @dbschemaversion int;
select top 1 @dbschemaversion = schema_version from dbo.[version] order by version_date desc;

if (@dbschemaversion < $(required_db_version))
begin
	raiserror ('DTLoggedExecDB must be updated to schema version $(required_db_version) or later before running this script.', 16, 127) with nowait
end

if (@dbschemaversion > $(required_db_version))
begin
	raiserror ('DTLoggedExecDB is already updated to a newer version.', 16, 127) with nowait
end
go

/*

	Check for compatibility

*/
set xact_abort on
go

begin tran
go

print 'Updating schema...'
go

create table dbo.labels
(
	label_id int not null identity(1,1),
	label varchar(512) not null
)
go

alter table dbo.labels
add constraint pk__labels primary key clustered (label_id)
go

set identity_insert dbo.labels on
go
insert into dbo.labels (label_id, label) values (-1, 'Undefined')
go
set identity_insert dbo.labels off
go

alter table dbo.package_executions
add label_id int not null default (-1)
go

alter table dbo.package_executions
add constraint fk__package_executions__labels foreign key(label_id) references dbo.labels(label_id)
go

alter view [dbo].[vw_executed_packages]
as
select 
	p.package_id,
	pe.package_execution_id,
	project_name = prj.name,
	section_name = s.name,
	package_name = p.name,
	package_version = p.version,
	package_guid,
	version_guid,
	execution_owner = user_name,
	execution_server = machine_name,
	execution_datetime,
	execution_label = l.label,
	load_start_datetime,
	load_end_datetime,
	load_elapsed_time_secs = DATEDIFF(ss, load_start_datetime, load_end_datetime),
	package_start_datetime,
	package_end_datetime,
	package_elapsed_time_secs = DATEDIFF(ss, package_start_datetime, package_end_datetime),
	load_result,
	package_result,
	warnings,
	errors
from
	dbo.packages p
inner join
	dbo.package_executions pe on p.package_id = pe.package_id
inner join
	dbo.package_execution_results per on per.package_execution_id = pe.package_execution_id
inner join
	dbo.projects prj on p.project_id = prj.project_id 
inner join
	dbo.sections s on p.[section_id] = s.[section_id]
inner join
	dbo.labels l on pe.[label_id] = l.[label_id]
go

insert into dbo.[version] values (19, '20110912')
go

commit tran
go

print 'Done!'
go
