﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;

namespace DTLoggedExec
{
    public class DataFlowProfileProvider : IDisposable
    {
        private const string EXTENSION = ".dtsDataFlowProfile";

        private object _lockHolder = new object();
        private StreamWriter _writer;
        private Dictionary<string, int> _dataFlowExecutionCount = new Dictionary<string, int>();
        private string _dataFlowProfilingFilePath = string.Empty;        

        public string ProfileFilePath
        {
            get { return _dataFlowProfilingFilePath; }
        }

        public DataFlowProfileProvider(Package package, string profilePath, string profileFileName, Guid executionGuid, Uri packageUri)
        {
            if (string.IsNullOrEmpty(profilePath) || string.IsNullOrEmpty(profileFileName))
            {
                throw new ArgumentException("profilePath and profileFileName cannot be empty");               
            }

            profilePath = Path.GetFullPath(profilePath);
            if (!Directory.Exists(profilePath))
            {
                throw new ArgumentException(string.Format("Directory referenced in profilePath must exist ({0})", profilePath));               
            }

            profileFileName = profileFileName.Replace("{date}", DateTime.Now.ToString("yyyyMMdd"));
            profileFileName = profileFileName.Replace("{time}", DateTime.Now.ToString("HHmmss"));
            profileFileName = profileFileName.Replace("{filename}", Path.GetFileNameWithoutExtension(packageUri.AbsolutePath));

            if (Path.GetExtension(profileFileName) != EXTENSION) profileFileName += EXTENSION;

            _dataFlowProfilingFilePath = Path.Combine(profilePath, profileFileName);

            _writer = new StreamWriter(_dataFlowProfilingFilePath, false, Encoding.UTF8, 512);

            _writer.WriteLine("#__DataFileVersion: 3");
            _writer.WriteLine("#Package: " + package.Name);
            _writer.WriteLine("#Version: {0}.{1}.{2}", package.VersionMajor, package.VersionMinor, package.VersionBuild);
            _writer.WriteLine("#PackageGUID: {0}", package.ID);
            _writer.WriteLine("#VersionGUID: {0}", package.VersionGUID);
            _writer.WriteLine("#ExecutionGUID: {{{0}}}", executionGuid.ToString().ToUpperInvariant());
            _writer.WriteLine("#UserName: {0}", Environment.UserName);
            _writer.WriteLine("#MachineName: {0}", Environment.MachineName);
            _writer.WriteLine("#ExecutionDateTime: {0}", DateTime.Now.ToString("s"));
            _writer.WriteLine("DataFlowId::DataFlowName::ExecutionNumber::EventName::EventDate::EventTime::SourceId::SourceName::ComponentId::ComponentName::DestinationId::DestinationName::RowCount");
        }        

        public void ProcessDataFlowData(TaskHost taskHost, string eventName, string eventText, object[] arguments, string subComponent)
        {
            if ((taskHost == null) || !(taskHost.InnerObject is MainPipe)) return;

            lock (_lockHolder)
            {
                string dataFlowId = taskHost.ID;
                if (eventName == "OnPreExecute")
                {
                    if (_dataFlowExecutionCount.ContainsKey(dataFlowId))
                    {
                        _dataFlowExecutionCount[dataFlowId] += 1;
                    }
                    else
                    {
                        _dataFlowExecutionCount[dataFlowId] = 1;
                    }
                }

                if (eventName == "OnPipelineRowsSent")
                {
                    _writer.Write(taskHost.ID + "::" + taskHost.Name + "::" + _dataFlowExecutionCount[dataFlowId].ToString() + "::" + eventName + "::" + DateTime.Now.ToString("yyyyMMdd") + "::" + DateTime.Now.ToString(@"HH\:mm\:ss\.fffffff"));

                    if (arguments.Length == 8)
                    {
                        for (int i = 1; i < 8; i ++)
                        {
                            _writer.Write("::" + arguments[i]);
                        }

                        _writer.WriteLine();
                    }
                    else
                    {
                        _writer.WriteLine("WARNING: unexpected format of 'arguments' data.");
                        _writer.WriteLine();
                        if (arguments != null)
                        {
                            foreach (object a in arguments)
                            {
                                _writer.WriteLine("\t" + a);
                            }
                        }
                    }
                }

            }
        }

        #region IDisposable Members

        public void Dispose()
        {
            _writer.Close();
        }

        #endregion
    }
}
