using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Data;
using System.Data.SqlClient;
using Microsoft.SqlServer.Dts.Runtime;
using CommandLine;
using System.IO;
using System.Reflection;
using DTLoggedExec.LogProviders;

namespace DTLoggedExec
{
    public class Program
    {
        const string PARAM_PREFIX = "param_";
        const string PARAM_SCHEMA = "Param";
        const int NO_ERROR_RETURN_VALUE = 0;
        const int ERROR_RETURN_VALUE = -1;

        private static Guid _executionGuid = Guid.NewGuid();
        private static Uri _packageUri = null;

        static int Main(string[] args)
        {
            ShowProgramInfo();

            ApplicationArguments appArgs = new ApplicationArguments();
            if (Parser.ParseArgumentsWithUsage(args, appArgs))
            {
                // Check that deprecated options does not confict with latest ones
                if (!string.IsNullOrEmpty(appArgs.ProfileDataFlowFileName))        
                {
                    if (!string.IsNullOrEmpty(appArgs.ProfilePath) || !string.IsNullOrEmpty(appArgs.ProfileFileName))
                    {
                        Console.WriteLine("Error in specifed parameters.");
                        Console.WriteLine("'ProfileDataFlowFileName' option cannot be used with 'ProfileDataFlow', 'ProfilePath' and 'ProfileFileName', since it replaces all of them.");
#if DEBUG
                        Console.ReadLine();
#endif
                        return ERROR_RETURN_VALUE;
                    }
                }

                // Set events to log
                string[] eventsToLog = appArgs.LogEvents.Split(new char[] {','});
                PackageEvents packageEventsToLog = GetPackageEventsToLog(eventsToLog);           

                // Load specified custom log provider
                string logProviderName = appArgs.LogProvider;
                Console.WriteLine("Requested log provider: " + logProviderName);
                Console.WriteLine();

                // Store package position                
                if (appArgs.FILE != null) _packageUri = new Uri("file://" + appArgs.FILE);
                if (appArgs.DTS != null)                
                {
                    Uri rootUri = new Uri("dts://");
                    if (!(Uri.TryCreate(rootUri, appArgs.DTS, out _packageUri)))
                    {
                        Console.WriteLine("Invalid value for \"DTS\" parameter.");
#if DEBUG
                        Console.ReadLine();
#endif
                        return ERROR_RETURN_VALUE;
                    }
                }                
                if (appArgs.SQL != null) {
                    Uri rootUri = new Uri("sql://");
                    if (!(Uri.TryCreate(rootUri, appArgs.SQL, out _packageUri)))
                    {
                        Console.WriteLine("Invalid value for \"SQL\" parameter.");
#if DEBUG
                        Console.ReadLine();
#endif
                        return ERROR_RETURN_VALUE;  
                    }
                }
                
                // Create custom event handler
                Console.WriteLine("Creating custom event handler...");
                if (appArgs.LogProviderArguments != null) appArgs.LogProviderArguments = appArgs.LogProviderArguments.Trim(); 
                CustomEventHandler ceh = CreateCustomEventHandler(logProviderName, packageEventsToLog, appArgs.LogProviderArguments);
                if (ceh == null)
                {
                    return ERROR_RETURN_VALUE;
                }
                Console.WriteLine("...Done.");
                Console.WriteLine();

                // Find where package is stored and how it has to be loaded

                // Load Package
                Application app = new Application();
                Package pkg = null;

                Console.WriteLine("Starting to load Package...");                 
                
                if (appArgs.Decrypt != null)
                    app.PackagePassword = appArgs.Decrypt;

                try
                {
                    if (appArgs.FILE == null && appArgs.DTS == null && appArgs.SQL == null)
                    {
                        throw new ApplicationException("Package not specified.");

                    }
                        if (appArgs.FILE != null)
                    {
                        Console.WriteLine("Package on file: " + appArgs.FILE);
                        Console.WriteLine();
                        ceh.FireLoadPackageStart();
                        pkg = app.LoadPackage(appArgs.FILE, ceh);
                    }

                    if (appArgs.DTS != null)
                    {
                        Console.WriteLine("Package on DTS Server: " + appArgs.DTS);
                        Console.WriteLine(); 
                        ceh.FireLoadPackageStart(); 
                        pkg = app.LoadFromDtsServer(appArgs.DTS, appArgs.Server, ceh);
                    }

                    if (appArgs.SQL != null)
                    {
                        Console.WriteLine("Package on SQL Server: " + appArgs.SQL);
                        Console.WriteLine(); 
                        ceh.FireLoadPackageStart();
                        pkg = app.LoadFromSqlServer(appArgs.SQL, appArgs.Server, appArgs.User, appArgs.Password, ceh);                        
                    }                    

                    ceh.FireLoadPackageEnd();                    
                }
                catch (Exception ex)
                {
                    ceh.FireLoadPackageEnd(ex);                                      
                }
                Console.WriteLine();

                if (pkg == null)
                {
                    ceh.Dispose();
                    Console.WriteLine("Cannot load package.");
                    if ((appArgs.FILE == null) && (appArgs.DTS == null) && (appArgs.SQL == null))
                        Console.WriteLine("At least one of the DTS, SQL, or File options must be specified.");
#if DEBUG
                    Console.ReadLine();
#endif      
                    return ERROR_RETURN_VALUE;
                }               

                // Show if profiling is enabled
                DataFlowProfileProvider dataFlowProfileProvider = null;
                if (!string.IsNullOrEmpty(appArgs.ProfileDataFlowFileName))
                {                   
                    appArgs.ProfileDataFlow = true;
                    appArgs.ProfilePath = Path.GetDirectoryName(Path.GetFullPath(appArgs.ProfileDataFlowFileName));
                    appArgs.ProfileFileName = Path.GetFileName(appArgs.ProfileDataFlowFileName);
                }
                Console.WriteLine("DataFlow Profiling enabled: " + appArgs.ProfileDataFlow.ToString());
                if (appArgs.ProfileDataFlow)
                {
                    dataFlowProfileProvider = new DataFlowProfileProvider(pkg, appArgs.ProfilePath, appArgs.ProfileFileName, _executionGuid, _packageUri);
                    Console.WriteLine("DataFlow Profile data file: " + dataFlowProfileProvider.ProfileFilePath);

                    ceh.AttachDataFlowProfiler(dataFlowProfileProvider);
                }

                // **** This code is commented to mimic DTExec behaviour.
                // **** TODO: Add a switch on command line to halt on errors or not
                //if (ceh.ErrorsCount > 0)
                //{
                //    Console.WriteLine("Errors found while loading package.");
                //    return ERROR_RETURN_VALUE;
                //}

                Console.WriteLine("Package version: {0}.{1}.{2}", pkg.VersionMajor, pkg.VersionMinor, pkg.VersionBuild);
                Console.WriteLine("...Done.");
                Console.WriteLine();

                // If ShowParam option has been choosen don't run the package but just show supported parameters
                if (appArgs.ShowParams)
                {
                    ShowPackageParams(pkg);
                    Console.WriteLine();
#if DEBUG
                    Console.ReadLine();
#endif
                    return NO_ERROR_RETURN_VALUE;
                }
                
                // Load runtime params
                string[] rparams = appArgs.Param;

                // Load runtime configured values
                string[] values = appArgs.Set;

                // Load runtime references to config files
                string[] configurations = appArgs.ConfigFile;

                // Load autoconfiguration
                AutoConfigurationData acd = AutoConfigurationData.Load(appArgs.AutoConfigure);

                // Set default result value
                DTSExecResult result = DTSExecResult.Canceled;

                // Execute Package
                Console.WriteLine("Waiting for package execution to start...");
                Console.WriteLine();
                result = ExecutePackage(pkg, configurations, values, rparams, acd, ceh);
                Console.WriteLine("...Package Executed.");
                Console.WriteLine();

                // Clean up
                ceh.Dispose();

                // Print package execution result
                Console.WriteLine("Result: " + result.ToString());
                Console.WriteLine();
#if DEBUG
                Console.ReadLine();
#endif

                // Return
                return (int)result;
            }
            else
            {
                Console.WriteLine();
                Console.WriteLine("Please note that command line arguments are CASE SENSITIVE!");                
                
#if DEBUG
                Console.ReadLine();
#endif
                
                return ERROR_RETURN_VALUE;
            }
        }

        private static void ShowPackageParams(Package package)
        {                       
            Console.WriteLine("Params supported by Package: ");
            foreach (Variable v in package.Variables)
            {
                if (v.Name.StartsWith(PARAM_PREFIX) || v.Namespace == PARAM_SCHEMA)
                {
                    string paramName;

                    if (v.Namespace == PARAM_SCHEMA)
                    {
                        paramName = v.Name;
                    }
                    else
                    { 
                        paramName = v.Name.Substring(PARAM_PREFIX.Length);
                    }

                    Console.WriteLine();
                    Console.WriteLine("\tName: {0}", paramName);
                    Console.WriteLine("\tDescription: {0}", v.Description);
                    Console.WriteLine("\tDefault Value: {0}", v.Value);
                    Console.WriteLine("\tType: {0}", v.DataType.ToString());
                }
            }
        }

        /*
         * Handles package execution
         */
        private static DTSExecResult ExecutePackage(Package package, string[] configurations, string[] values, string[] rparams, AutoConfigurationData autoConfigData, CustomEventHandler customEventHandler)
        {
            // Show configurations specified in the package           
            Console.WriteLine("Package has configurations support enabled: " + package.EnableConfigurations.ToString());
            if ((package.Configurations.Count > 0) && (package.EnableConfigurations))
            {
                Console.WriteLine("Package references and tries to use {0} configuration(s):", package.Configurations.Count);
                foreach (Configuration config in package.Configurations)
                {
                    Console.WriteLine("\t{0}: {1}", config.Name, config.ConfigurationString);
                }
            }           

            // Show Auto-Configurations
            Console.WriteLine();
            Console.WriteLine("Checking for Auto-Configurations...");
            if (autoConfigData != null)
            {
                Console.WriteLine("Auto Configuration enabled. Configuration String is: \"" + autoConfigData.ConfigurationString + "\"");

                // Remove Design-Time configuration in order to allow "Auto Configuration" feature
                if (package.EnableConfigurations == false)
                {
                    Console.WriteLine("Removing Design-Time Configurations to avoid inconsistencies...");
                    int totalConfigurations = package.Configurations.Count;
                    for (int c = 0; c < totalConfigurations; c++)
                    {
                        package.Configurations.Remove(0);
                    }
                }

                Console.WriteLine("Adding Auto Configuration objects...");

                ConnectionManager cfgCM = package.Connections.Add("OLEDB");               
                cfgCM.Name = "$$DTLoggedExec_AutoConfigure";
                cfgCM.ConnectionString = string.Format("Data Source={0};Initial Catalog={1};Provider=SQLNCLI10;Integrated Security=SSPI;Auto Translate=False;", autoConfigData.SqlServerInstance, autoConfigData.Database);

                Configuration cfgGlobal = package.Configurations.Add();
                cfgGlobal.Name = "$$DTLoggedExec_Global";
                cfgGlobal.ConfigurationType = DTSConfigurationType.SqlServer;
                cfgGlobal.ConfigurationString = string.Format(@"""$$DTLoggedExec_AutoConfigure"";""{0}"";""$$Global"";", autoConfigData.Table);

                Configuration cfgLocal = package.Configurations.Add();
                cfgLocal.Name = "$$DTLoggedExec_Local";
                cfgLocal.ConfigurationType = DTSConfigurationType.SqlServer;
                cfgLocal.ConfigurationString = string.Format(@"""$$DTLoggedExec_AutoConfigure"";""{0}"";""{1}"";", autoConfigData.Table, package.Name);

                package.EnableConfigurations = true;
                Console.WriteLine("Auto Configuration Settings:");
                foreach (Configuration config in package.Configurations)
                {
                    if (config.Name.StartsWith("$$DTLoggedExec_"))
                    {
                        Console.WriteLine("\t{0}: {1}", config.Name, config.ConfigurationString);
                    }
                }
            }

            // Load configurations
            Console.WriteLine();
            Console.WriteLine("Loading additional configurations specified in command line...");
            foreach (string configFile in configurations)
            {
                if (File.Exists(configFile))
                {
                    Console.WriteLine("\tLoading configuration: " + configFile);
                    try
                    {
                        //package.SuppressConfigurationWarnings = false;
                        package.ImportConfigurationFile(configFile);

                        if (package.Warnings.Count > 0)
                        {
                            Console.WriteLine("\tWarnings:");
                            foreach (DtsWarning warning in package.Warnings)
                            {
                                Console.WriteLine("\t\t{0}", warning.Description);
                            }
                        }

                        if (package.Errors.Count > 0)
                        {
                            Console.WriteLine("\tErrors:");
                            foreach (DtsError error in package.Errors)
                            {
                                Console.WriteLine("\t\t{0}", error.Description);
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine("\tUnable to load configuration file.");
                        Console.WriteLine("\tException trapped:");
                        Console.WriteLine("\t\t" + ex.Message);

                        foreach (DtsWarning w in package.Warnings)
                        {
                            Console.WriteLine("\t\t{0}", w.Description);
                        }
                                                
                        return DTSExecResult.Failure;
                    }
                }
                else
                {
                    Console.WriteLine("\tConfiguration file '" + configFile + "' not found.");
                }
            }
            Console.WriteLine("...Done.");

            // Set values read from command line
            Console.WriteLine();
            Console.WriteLine("Configuring package with commandline-specified values...");
            Console.WriteLine("DTLoggedExec Parameters:");
            SetPackageValues(package, rparams, true);
            Console.WriteLine("SSIS Set Value:");
            SetPackageValues(package, values, false);
            Console.WriteLine("...Done.");

            // Detect and Display Connection Managers used in package
            Console.WriteLine(); 
            Console.WriteLine("Connection Managers used in package (please note that values changed through auto-configuration are not visible at this stage):");            
            foreach (ConnectionManager cm in package.Connections)
            {                
                Console.WriteLine("\t{0}: {1}", cm.Name, cm.ConnectionString);
            }

            // Store Warning & Errors so far
            string preExecuteWarnings = string.Empty;
            string preExecuteErrors = string.Empty;
            if (package != null)
            {
                preExecuteWarnings = package.Warnings.Count.ToString();
                preExecuteErrors = package.Errors.Count.ToString();
            }

            // Get Start Time
            Console.WriteLine();
            Console.WriteLine("Executing...");
            DateTime startTime = DateTime.Now;

            // Execute package binding the customized event handler
            Console.WriteLine();
            DTSExecResult result = DTSExecResult.Failure;
            try
            {
                customEventHandler.FirePackageStart(package);

                result = package.Execute(null, null, customEventHandler, null, null);

                customEventHandler.FirePackageEnd(package, result);
            }
            catch (DtsException de)
            {
                customEventHandler.FirePackageEnd(package, result, de);
            }

            // Get End Time
            DateTime endTime = DateTime.Now;

            // Store Warning & Errors so far
            string postExecuteWarnings = string.Empty;
            string postExecuteErrors = string.Empty;
            if (package != null)
            {
                postExecuteWarnings = package.Warnings.Count.ToString();
                postExecuteErrors = package.Errors.Count.ToString();
            }

            // Calculate Elapsed Time
            TimeSpan elapsedTime = endTime - startTime;
            Console.WriteLine();
            Console.WriteLine("Total warnings (load/execution/events): {0}/{1}/{2}", preExecuteWarnings, postExecuteWarnings, customEventHandler.WarningsCount.ToString());
            Console.WriteLine("Total errors (load/execution/events): {0}/{1}/{2}", preExecuteErrors, postExecuteErrors, customEventHandler.ErrorsCount.ToString());          
            Console.WriteLine("Elapsed time: " + elapsedTime.TotalSeconds.ToString() + " secs.");

            return result;
        }

        /*
         * Creates the custom event handler that intercepts ssis events.
         * All intercepted events are directed to the user-specified log provider
         */
        private static CustomEventHandler CreateCustomEventHandler(string logProviderTypeName, PackageEvents packageEventsToLog, string logProviderConfigurationString)
        {
            // Instantiate custom log provider
            ILogProvider logProvider = null;
            try
            {
                Assembly providerAssembly = Assembly.Load(logProviderTypeName);

                Type logProviderType = providerAssembly.GetType(logProviderTypeName);
                if (logProviderType == null)
                {
                    throw new ApplicationException(string.Format("Log provider \"{0}\" not found.", logProviderTypeName));
                }
                Console.WriteLine("Loaded from assembly: " + providerAssembly.FullName);

                logProvider = (ILogProvider)Activator.CreateInstance(logProviderType);

                logProvider.ShowVersionInformation();
                logProvider.SetExecutionGuid(_executionGuid);
                logProvider.SetPackageInfo(_packageUri);

                Console.WriteLine("Configuring log provider....");
                logProvider.Configure(logProviderConfigurationString);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception trapped: " + ex.Message);
                return null;
            }

            // Instantiate custom package event handler 
            // and bind it to the chosen custom log provider
            CustomEventHandler ceh = new CustomEventHandler(logProvider, packageEventsToLog);

            return ceh;
        }

        private static DTSExecResult SetPackageValues(Package package, string[] values, bool parseAsParameter)
        {
            int value_count = 0;

            foreach (string v in values)
            {
                string valuePath = string.Empty;
                string[] s = null;

                if (parseAsParameter)
                {                    
                    s = v.Split(';');
                    string paramName = s[0].Trim();

                    DtsProperty dummy = null;
                    
                    string valuePath1 = valuePath1 = "\\Package.Variables[User::" + PARAM_PREFIX + paramName + "].Properties[Value]";
                    string valuePath2 = valuePath2 = "\\Package.Variables[" + PARAM_SCHEMA + "::" + paramName + "].Properties[Value]";

                    bool usesPrefix = false;
                    bool usesSchema = false;

                    try
                    {
                        usesPrefix = (package.GetObjectFromPackagePath(valuePath1, out dummy) != null);
                        valuePath = valuePath1;
                    }
                    catch (DtsRuntimeException)
                    {
                        usesPrefix = false;
                    }

                    try
                    {
                        usesSchema = (package.GetObjectFromPackagePath(valuePath2, out dummy) != null);
                        valuePath = valuePath2;
                    }
                    catch (DtsRuntimeException)
                    {
                        usesSchema = false;
                    }
                   
                    if (usesPrefix == false && usesSchema == false)
                    {
                        Console.WriteLine("\tError: Parameter \"{0}\" does not exists not as \"{1}\" nor as \"{2}\".", paramName, valuePath1, valuePath2);
                        return DTSExecResult.Failure;
                    }

                    if (usesPrefix == true && usesSchema == true)
                    {
                        Console.WriteLine("\tError: Ambiguity while setting parameter \"{0}\". Both variant \"{1}\" and \"{2}\" exists.", paramName, valuePath1, valuePath2);
                        return DTSExecResult.Failure;
                    }                                                                
                }
                else
                {
                    s = v.Split(';');
                    valuePath = s[0];
                }

                if (s.GetUpperBound(0) < 1)
                {
                    Console.WriteLine("\tError: No value specified for '{0}'. Set options supports only the 'path;value' form.", valuePath);
                    return DTSExecResult.Failure;
                }

                Console.Write("\t{0} = {1}", valuePath, s[1]);

                try
                {
                    DtsProperty p;
                    Variable pkgObj;
                    pkgObj = (Variable)package.GetObjectFromPackagePath(valuePath, out p);                                        

                    if (p != null && pkgObj != null)
                    {
                        Console.WriteLine(" (Parameter Type: " + pkgObj.DataType + ")");
                        p.SetValue(pkgObj, Convert.ChangeType(s[1], pkgObj.DataType));
                    }
                    else
                    {
                        Console.WriteLine();
                        if (p == null)
                        {
                            Console.WriteLine("\tError: Property \"{0}\" does not exists.", valuePath);
                            return DTSExecResult.Failure;
                        }
                        if (pkgObj == null)
                        {
                            Console.WriteLine("\tError: Cannot find package object requested in \"{0}\" does not exists.", valuePath);
                            return DTSExecResult.Failure;
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine("\tError while setting value for \"{0}\".", valuePath);
                    Console.WriteLine("\tException trapped: " + e.Message);
                    return DTSExecResult.Failure;
                }

                value_count++;
            }

            if (value_count == 0)
            {
                Console.WriteLine("\tNo values found.");
            }

            return DTSExecResult.Completion;
        }

        /*
         * Set which events will be logged
         */
        private static PackageEvents GetPackageEventsToLog(string[] eventsToLog)
        {
            PackageEvents result = PackageEvents.None;

            if ((Array.IndexOf<string>(eventsToLog, "C") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {                
                result |= PackageEvents.Custom;
            }

            if ((Array.IndexOf<string>(eventsToLog, "E") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.Error;
            }

            if ((Array.IndexOf<string>(eventsToLog, "ESC") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.ExecutionStatusChanged;
            }

            if ((Array.IndexOf<string>(eventsToLog, "I") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.Information;
            }

            if ((Array.IndexOf<string>(eventsToLog, "POE") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.PostExecute;
            }

            if ((Array.IndexOf<string>(eventsToLog, "POV") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.PostValidate;
            }

            if ((Array.IndexOf<string>(eventsToLog, "PRE") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.PreExecute;
            }

            if ((Array.IndexOf<string>(eventsToLog, "PRV") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.PreValidate;
            }

            if ((Array.IndexOf<string>(eventsToLog, "P") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.Progress;
            }

            if ((Array.IndexOf<string>(eventsToLog, "T") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.TaskFailed;
            }

            if ((Array.IndexOf<string>(eventsToLog, "W") >= 0) || (Array.IndexOf<string>(eventsToLog, "*") >= 0))
            {
                result |= PackageEvents.Warning;
            }

            Console.WriteLine("Logged Events: " + result.ToString());

            return result;
        }       

        /*
         * Show program and authors informations
         */
        private static void ShowProgramInfo()
        {
            Assembly executingAssembly = System.Reflection.Assembly.GetExecutingAssembly();
            string copyright = ((AssemblyCopyrightAttribute)executingAssembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), true)[0]).Copyright;
            string version = executingAssembly.GetName().Version.ToString();
           
            Console.WriteLine("DTLoggedExec " + copyright + " v." + version );
            Console.WriteLine("Some Rights Reserved. Licensed under the Creative Commons Attribution-NonCommercial-NoDerivs 2.5 Italy License.");
            Console.WriteLine("More info here: http://dtloggedexec.davidemauri.it");
            Console.WriteLine();
            //Console.WriteLine("Beta version, use at your own risk!");
            //Console.WriteLine();
            Console.WriteLine("Target platform: " + (IntPtr.Size * 8).ToString() + "bit");
            Console.WriteLine();           
        }
    }
}
